<?php

namespace Litecms\Testimonial\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Litecms\Testimonial\Http\Requests\TestimonialRequest;
use Litecms\Testimonial\Interfaces\TestimonialRepositoryInterface;
use Litecms\Testimonial\Models\Testimonial;
use Litecms\Testimonial\Forms\Testimonial as Form;

/**
 * APIController  class for testimonial.
 */
class TestimonialAPIController extends BaseController
{

    /**
     * Initialize testimonial resource controller.
     *
     * @param type TestimonialRepositoryInterface $testimonial
     *
     * @return null
     */
    public function __construct(TestimonialRepositoryInterface $testimonial)
    {
        parent::__construct();
        $this->repository = $testimonial;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Testimonial\Repositories\Criteria\TestimonialResourceCriteria::class);
    }

    /**
     * Display a list of testimonial.
     *
     * @return Response
     */
    public function index(TestimonialRequest $request)
    {
        return $this->repository
            ->setPresenter(\Litecms\Testimonial\Repositories\Presenter\TestimonialPresenter::class)
            ->paginate();
    }

    /**
     * Display testimonial.
     *
     * @param Request $request
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function show(TestimonialRequest $request, Testimonial $testimonial)
    {
        return $testimonial->setPresenter(\Litecms\Testimonial\Repositories\Presenter\TestimonialListPresenter::class);
        ;
    }

    /**
     * Create new testimonial.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TestimonialRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('testimonial::testimonial.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('testimonial/testimonial/' . $testimonial->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('testimonial/testimonial');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the testimonial.
     *
     * @param Request $request
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function update(TestimonialRequest $request, Testimonial $testimonial)
    {
        try {
            $data = $request->all();

            $testimonial->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('testimonial::testimonial.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('testimonial/testimonial/' . $testimonial->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('testimonial/testimonial/' . $testimonial->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the testimonial.
     *
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function destroy(TestimonialRequest $request, Testimonial $testimonial)
    {
        try {
            $testimonial->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('testimonial::testimonial.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('testimonial/testimonial/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('testimonial/testimonial/' . $testimonial->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
